package flyweight_demo;

import java.util.HashMap;
import java.util.Map;

import visitor_demo.book.Text;
import visitor_demo.book.TextVisitor;

/**
 * Word is an immutable object represent a single word. Commenting the class
 * properly is left as an exercise for the reader.
 * 
 * @author Krysta Yousoufian, CSE 331
 * 
 */
public class FlyweightWord implements Text {
    // This works because words are immutable! If they're not, you can't intern!
    private static Map<String, FlyweightWord> CACHED_WORDS = new HashMap<String, FlyweightWord>();

    String value;

    public static FlyweightWord getWord(String value) {
        if (CACHED_WORDS.containsKey(value)) {
            return CACHED_WORDS.get(value);
        } else {
            FlyweightWord w = new FlyweightWord(value);
            CACHED_WORDS.put(value, w);
            return w;
        }
    }

    private FlyweightWord(String value) {
        this.value = value;
    }

    /**
     * @param previous
     *            the preceding word, or null if this is the first word in the
     *            passage.
     * @param next
     *            the following word, or null if this is the last word in the
     *            passage.
     * @return the three-word window of text to which this Word belongs, i.e.
     *         the word before, this word, and the word after. If there is no
     *         word before or after, it omits the nonexistent word.
     */
    public String printWindow(FlyweightWord previous, FlyweightWord next) {
        StringBuilder builder = new StringBuilder();
        if (previous != null)
            builder.append(previous.value + " ");
        builder.append(value);
        if (next != null)
            builder.append(" " + next.value);
        return builder.toString();
    }

    public String getValue() {
        return value;
    }

    @Override
    public void accept(TextVisitor v) {
        v.visit(this);
    }

    @Override
    public boolean equals(Object o) {
        if (o == null)
            return false;
        else if (o instanceof FlyweightWord)
            return ((FlyweightWord) o).value == this.value;
        else
            return false;
    }

    @Override
    public int hashCode() {
        return value.hashCode();
    }

}
