package flyweight_demo;

import visitor_demo.book.Text;
import visitor_demo.book.TextVisitor;

/**
 * Word is an immutable object representing a single word. Commenting the class
 * properly is left as an exercise for the reader.
 * 
 * @author Krysta Yousoufian, CSE 331
 * 
 */
public class NonFlyweightWord implements Text {
    // The word itself.
    String value;
    // Previous and next word in a block of text, forming a chain of Words.
    NonFlyweightWord previous, next;

    public NonFlyweightWord(String value, NonFlyweightWord previous,
            NonFlyweightWord next) {
        this.value = value;
        this.previous = previous;
        this.next = next;
    }

    /**
     * @return the three-word window of text to which this Word belongs, i.e.
     *         the word before, this word, and the word after. If there is no
     *         word before or after, it omits the nonexistent word.
     */
    public String printWindow() {
        StringBuilder builder = new StringBuilder();
        if (previous != null)
            builder.append(previous.value + " ");
        builder.append(value);
        if (next != null)
            builder.append(" " + next.value);
        return builder.toString();
    }

    public String getValue() {
        return value;
    }

    @Override
    public void accept(TextVisitor v) {
        v.visit(this);
    }

    // This should be updated to reflect new fields.
    @Override
    public boolean equals(Object o) {
        if (o == null)
            return false;
        else if (o instanceof NonFlyweightWord)
            return ((NonFlyweightWord) o).value == this.value;
        else
            return false;
    }

    // This should be updated to reflect new fields.
    @Override
    public int hashCode() {
        return value.hashCode();
    }
}
