package interning_demo;

import java.util.HashMap;
import java.util.Map;

import visitor_demo.book.Text;
import visitor_demo.book.TextVisitor;

/**
 * Word is an immutable object representing a single word.
 * Commenting the class properly is left as an exercise for the reader.
 * @author Krysta Yousoufian, CSE 331
 *
 */
public class Word implements Text {
    // This works because words are immutable! If they're not, you can't intern!
    private static Map<String, Word> CACHED_WORDS = new HashMap<String, Word>();
    
    String value;
    
    public static Word getWord(String value) {
        if (CACHED_WORDS.containsKey(value)) {
            return CACHED_WORDS.get(value);
        } else {
            Word w = new Word(value);
            CACHED_WORDS.put(value, w);
            return w;
        }
    }
    
    private Word(String value) {
        this.value = value;
    }
    
    public String getValue() {
        return value;
    }

    @Override
    public void accept(TextVisitor v) {
        v.visit(this);
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == null)
            return false;
        else if (o instanceof Word)
            return ((Word)o).value == this.value;
        else
            return false;
    }
    
    @Override
    public int hashCode() {
        return value.hashCode();
    }

}
