package singleton_demo;

import java.io.File;

/**
 * Handles a single client's file read requests.
 * Repeatedly accepts the next incoming request for some file. A real
 * server would load the requested file and send it back to the client.
 * Our fake server implementation doesn't actually do anything except
 * log whether the file was found. How silly!
 * 
 * @author Krysta Yousoufian, CSE 331
 *
 */
public class ClientHandler {
	private Logger logger;
	
	public ClientHandler() {
		logger = Logger.getInstance(); 
	}
	
	/**
	 * Continually process requests for files. (In a real server these would
   * come over the network; here, we read them from the console.)
	 */
	public void readNextFile() {
		String fname = IOUtil.getFilename();
		File f = new File(fname);
		if (f.exists()) {
			// Here we would open the file, read it, and send the contents
			// back to the client
			logger.write("Read file: " + fname);
		} else {
			logger.write("ERROR: File not found: " + fname);
		}
	}
}
